/*
 * KeggGene.h
 *
 *  Created on: 2013/01/09
 *      Author: Axiohelix
 */

#ifndef KeggGene_H_
#define KeggGene_H_

#include	<string>
#include	<vector>
#include	<map>
#include	<algorithm>

#include	"TaxonomyNode.h"
#include	"KO.h"

class USCG;

/**
 * KEGG genesの情報を格納するオブジェクト.
 */
class KeggGene {
	private:
		/* KEGG ID */
		std::string m_id;

		/* length */
		int m_length;

		/* Taxonomy information */
		TaxonomyNode* m_taxNode;

		/* KO information */
		std::vector<KO *>* m_koVec;

		/* カウンタ */
		int m_count;
		double m_norm;

	public:
		/* コンストラクタ・デストラクタ */
		KeggGene() { m_taxNode = NULL; m_koVec = NULL; }

		KeggGene(const std::string id, const int len) {
			m_id = id;
			m_length = len;

			m_count = 0;
			m_norm = 0.0;
			m_taxNode = NULL;
			m_koVec = NULL;
		}

		virtual ~KeggGene();

		/* setter / getter */
		inline std::string GetId() const { return m_id; }

		inline int GetLength() const { return m_length; }

		void SetTaxonomy(int, std::map<int, int>*, std::map<int, TaxonomyNode>*);
		inline TaxonomyNode* GetTaxNode() { return m_taxNode; }

		void SetKeggOrtholog(std::string, std::map<std::string, KO>*, std::map<std::string, USCG*>*);
		void AddKo(KO*);
		inline std::vector<KO *>* GetKoVec() { return m_koVec; }

		inline void SetCount(const int initial) { m_count = initial; }
		inline void inc() { m_count++; }
		inline int GetCount() const { return m_count; }

		inline void SetNorm(const double norm) { m_norm = norm; }
		inline double GetNorm() const { return m_norm; }
};

#endif /* KeggGene_H_ */
