/*
 * ParameterObj.cpp
 *
 *  Created on: 2011/01/07
 *      Author: Wings
 */

#include	<fstream>
#include	<sstream>

#include	"ParameterObj.h"

/* コンストラクタ */
ParameterObj::ParameterObj() {
}

/* デストラクタ */
ParameterObj::~ParameterObj() {
	// TODO Auto-generated destructor stub
}

/**
 * コマンドラインパラメータをパースする.
 */
void ParameterObj::ParseParameter(const int argc, const char *argv[]) {

	bool isValid = true;

	/* argv[0] はプログラム名 */
	for(int i=1; i<argc; i++) {
		if(!fileExists(i, std::string(argv[i]))) {
			SetErrorMessage(i + "th file \"" + std::string(argv[i]) + "\" doesn't exist or can't be opened.");
			isValid = false;
		}

		/* 順番に */
		switch (i) {
			case 1:
				setFileName(GENES_LIST, std::string(argv[i]));
				break;
			case 2:
				setFileName(GI_TAXID, std::string(argv[i]));
				break;
			case 3:
				setFileName(ROOT_MAP, std::string(argv[i]));
				break;
			case 4:
				setFileName(KO_ENZYME, std::string(argv[i]));
				break;
			case 5:
				setFileName(USCG_LIST, std::string(argv[i]));
				break;
			case 6:
				setFileName(BLAST_RESULT, std::string(argv[i]));
				break;
			case 7:
				setBlastThresholdScore(atoi(argv[i]));
				break;
			case 8:
				setBlastThresholdIdentity(atoi(argv[i]));
				break;
			case 9:
				setFileName(NORMALIZE_COUNT, std::string(argv[i]));
				break;
			case 10:
				setFileName(USCG_COUNT, std::string(argv[i]));
				break;
			case 11:
				setFileName(NORMALIZE_ERROR, std::string(argv[i]));
				break;
			case 12:
				setFileName(BEFORE_NORMALIZE, std::string(argv[i]));
				break;
			case 13:
				setFileName(NORMALIZE_OUT, std::string(argv[i]));
				break;
			default:
				SetErrorMessage("Too many arguments. : \"" + std::string(argv[i]) + "\"");
				isValid = false;
		}
	}

	/* すべてのファイルの指定が必須 */
	if(argc != 14) {
		SetErrorMessage("Too few arguments.");
		isValid = false;
	}

	SetIsValid(isValid);
}

const bool ParameterObj::fileExists(const int i, const std::string fileName) {
	if(i > 6) {
		return true;
	}

	std::ifstream input(fileName.c_str());

	return input;
}

/**
 * プログラムの使い方の文字列を生成する.
 */
const std::string ParameterObj::GetUsage(const std::string progName) {
	std::stringstream usage;

	usage << "Usage:" << LINE_DELIMITER;
	usage << "  " << progName << " params..." << LINE_DELIMITER;
	usage << "    params consist in " << LINE_DELIMITER;
	usage << "      1st : path to input file \"gi_taxid_prot.dmp\"" << LINE_DELIMITER;
	usage << "      2nd : path to input file \"root_map.txt\"" << LINE_DELIMITER;
	usage << "      3rd : path to input file \"input.fasta\"" << LINE_DELIMITER;
	usage << "      4th : path to input file \"blast_result.txt\"" << LINE_DELIMITER;
	usage << "      5th : threshold of score(integer)" << LINE_DELIMITER;
	usage << "      6th : threshold of identity(integer)" << LINE_DELIMITER;
	usage << "      7th : path to output file \"high_quality.fasta\"" << LINE_DELIMITER;
	usage << "      8th : path to output file \"filtered_eukaryote.fasta\"" << LINE_DELIMITER;
	usage << "      9th : path to output file \"filtered_taxonomy_error.fasta\"" << LINE_DELIMITER;
	if(GetErrorMessage().length() > 0) {
		usage << "Errors:" << LINE_DELIMITER;
		usage << GetErrorMessage() << LINE_DELIMITER;
	}

	return usage.str();
}
