/*
 * ParameterObj.h
 *
 *  Created on: 2011/01/07
 *      Author: Wings
 */

#ifndef PARAMETEROBJ_H_
#define PARAMETEROBJ_H_

#include	<string>
#include	<stdlib.h>

#include	<map>

#include	"Const.h"

/**
 * コマンドラインパラメータを格納するオブジェクト.
 */
class ParameterObj {
	public:
		enum FILE_TYPE {
			GENES_LIST,
			KO_ENZYME,
			USCG_LIST,

			GI_TAXID,
			ROOT_MAP,

			BLAST_RESULT,

			NORMALIZE_COUNT,
			USCG_COUNT,
			NORMALIZE_ERROR,
			BEFORE_NORMALIZE,
			NORMALIZE_OUT
		};

	private:
		std::map<FILE_TYPE, std::string> m_file_map;

		/* BLAST結果のscoreの閾値. */
		int m_blastThresholdScore;
		/* BLAST結果のidentityの閾値. */
		int m_blastThresholdIdentity;

		/* パース時のエラーメッセージ. */
		std::string m_errorMessage;
		/* パース時にエラーが発生したかどうか. */
		bool m_isValid;

	public:
		/* コンストラクタ・デストラクタ */
		ParameterObj();
		virtual ~ParameterObj();

		/* getter / setter */
		inline std::string getFileName(FILE_TYPE type) { return m_file_map.find(type) == m_file_map.end() ? "" : m_file_map[type]; }
		inline void setFileName(FILE_TYPE type, std::string filename) { m_file_map[type] = filename; }

		inline int getBlastThresholdScore() const { return m_blastThresholdScore; }
		inline void setBlastThresholdScore(const int score) { m_blastThresholdScore = score; }

		inline int getBlastThresholdIdentity() const { return m_blastThresholdIdentity; }
		inline void setBlastThresholdIdentity(const int identity) { m_blastThresholdIdentity = identity; }

		/* エラーメッセージをセットする. */
		/* 複数のエラー出力を行うため、追記式とする. */
		inline void SetErrorMessage(const std::string errorMessage) {
			if(m_errorMessage.size() > 0) {
				m_errorMessage += std::string(LINE_DELIMITER);
			}
			m_errorMessage += "  ";
			m_errorMessage += errorMessage;
		};
		inline const std::string GetErrorMessage() { return m_errorMessage; };

		inline void SetIsValid(const bool isValid) { m_isValid = isValid; };
		inline const bool IsValid() { return m_isValid; };

		void ParseParameter(const int, const char **);

		const std::string GetUsage(const std::string);

	private:
		const bool fileExists(const int, const std::string);
};

#endif /* PARAMETEROBJ_H_ */
