/*
 * Splitter.cpp
 *
 *  Created on: 2010/12/08
 *      Author: Axiohelix
 */

#include "Splitter.h"

/**
 * デフォルトコンストラクタ
 */
Splitter::Splitter() {
}

/**
 * コンストラクタ2.
 * 分割対象文字列とデリミタを1つだけ指定する.
 */
Splitter::Splitter(const std::string str, const std::string delimiter) {
	setStr(str, delimiter);
}

/**
 * コンストラクタ3.
 * 分割対象文字列とデリミタを複数指定する.
 */
Splitter::Splitter(const std::string str, const std::vector<std::string> delimVector) {
	setStr(str, delimVector);
}

/**
 * デストラクタ
 */
Splitter::~Splitter() {
	// TODO Auto-generated destructor stub
}

/**
 * 分割対象文字列をセットする.
 * 実処理はsetStr(sting, string)に任せる.
 */
void Splitter::setStr(const std::string str) {
	setStr(str, "\t");
}

/**
 * 分割対象文字列とデリミタを1つだけセットする.
 * 実処理はsetStr(sting, vector<string>)に任せる.
 */
void Splitter::setStr(const std::string str, const std::string delimiter) {
	std::vector<std::string> delimVector;
	delimVector.push_back(delimiter);

	setStr(str, delimVector);
}

/**
 * 分割対象文字列とデリミタを複数セットする.
 * その他いろいろな変数を初期化する.
 */
void Splitter::setStr(const std::string str, const std::vector<std::string> delimVector) {
	/* 分割開始位置は初期値0 */
	m_startPos = 0;
	/* 分割対象文字列をセット */
	m_str = str;
	/* 分割デリミタをセット */
	m_delimVector = delimVector;

	/* まだ分割は終わってない */
	m_isEnd = false;
}

/**
 * 一気にデリミタを用いて文字列を分解する.
 * このメソッドより先にsetStrのいずれかを呼んでおく必要がある.
 * 内部処理はほぼnext()に任せる.
 */
std::vector<std::string> Splitter::split() {
	/* 分割された文字列のvector */
	std::vector<std::string> wordVector;

	/* 分割が完了するまでループする. */
	while(!m_isEnd) {
		/* 分割された文字列1つを格納 */
		wordVector.push_back(next());
	}

	/* vectorに保持される分割された文字列を返す. */
	return wordVector;
}

/**
 * 現在の位置から次までの分割された文字列を取得する.
 */
std::string Splitter::next() {
	/* 既に分割が完了しているのなら常に空文字を返す. */
	if(m_isEnd) {
		return "";
	}

	/* デリミタの位置. */
	/* 初期値は文字列の最後(+1). */
	size_t delimPos = m_str.length();
	/* 分割に使用したデリミタ. */
	std::string delimiter = "";

	/* 複数のデリミタを全部試し、一番前方にヒットしたデリミタを使用する. */
	std::vector<std::string>::iterator iterator = m_delimVector.begin();
	for( ; iterator != m_delimVector.end(); iterator++) {
		std::string _delimiter = *iterator;
		size_t _pos = m_str.find(_delimiter, m_startPos);

		/* そのデリミタが分割対象文字列に存在し、かつ今までのデリミタの出現位置より前なら、*/
		if(_pos != std::string::npos and _pos < delimPos) {
			/* デリミタ情報を更新する */
			delimPos = _pos;
			delimiter = _delimiter;
		}
	}

	/* 今回の分割開始位置のバックアップ */
	int startPos = m_startPos;

	/* 次回の分割開始位置をセットしておく */
	m_startPos = delimPos + delimiter.length();
	/* 次回の分割開始位置が文字列を超えているのなら分割は終了したとする. */
	if(m_startPos >= m_str.length()) {
		m_isEnd = true;
	}

	/* 今回の分割開始位置から分割終了位置までを切り出し返す. */
	return m_str.substr(startPos, delimPos - startPos);
}
