/*
 * TaxonomyNode.cpp
 *
 *  Created on: 2010/12/07
 *      Author: Axiohelix
 */

#include <sstream>
#include <list>

#include "TaxonomyNode.h"

/**
 * このノードの情報表示を行うメソッド.
 * Lineageを表示するか、名前解決をするか、の指定により出力内容が変化する.
 */
const std::string TaxonomyNode::GetInformation(const bool isShowLineage, const bool isNameResolve) {
	std::stringstream str;

	/* IDの出力 */
	/* もし名前解決オプションが指定されているのなら名前を表示するが、-1のときは"undefined"で表示する */
	str << m_id << (isNameResolve ? ":" + (m_id > 0 ? m_name : "undefined") : "") << OUTPUT_DELIMITER;
	/* ルートのIDの出力 */
	/* もし名前解決オプションが指定されているのなら名前を表示するが、-1のときは"undefined"で表示する */
	str << m_rootId << (isNameResolve ? ":" + (m_rootId > 0 ? m_rootName : "undefined") : "") << OUTPUT_DELIMITER;
	/* 門のIDの出力 */
	/* もし名前解決オプションが指定されているのなら名前を表示するが、-1のときは"undefined"で表示する */
	str << m_phylumId << (isNameResolve ? ":" + (m_phylumId > 0 ? m_phylumName : "undefined") : "") << OUTPUT_DELIMITER;
	/* 属のIDの出力 */
	/* もし名前解決オプションが指定されているのなら名前を表示するが、-1のときは"undefined"で表示する */
	str << m_genusId << (isNameResolve ? ":" + (m_genusId > 0 ? m_genusName : "undefined") : "") << OUTPUT_DELIMITER;
	/* rankの出力 */
	str << m_rank;

	/* Lineage表示オプションが指定されている場合 */
	if(isShowLineage) {
		/* デリミタを出力し、Lineageを出力する */
		str << OUTPUT_DELIMITER;
		str << getLineage(LINEAGE_DELIMITER);
	}

	return str.str();
}

/**
 * そのノードのLineageを出力するメソッド.
 */
const std::string TaxonomyNode::getLineage(const std::string lineageDelimiter) {
	std::stringstream str;

	/* m_lineageListには逆順(子孫から祖先へ)にデータが格納されているため、 */
	/* 逆順のiteratorを利用する */
	std::list<std::string>::reverse_iterator iterator = m_lineageList.rbegin();
	bool isFirst = true;
	while(iterator != m_lineageList.rend()) {
		/* 出力が最初なら */
		if(isFirst) {
			/* 何もしない */
			isFirst = false;
		} else {
			/* lineageのデリミタを出力する */
			str << lineageDelimiter;
		}

		/* ノードの情報を出力する. */
		str << *iterator;
		iterator++;
	}

	return str.str();
}
