/*
 * TaxonomyNode.h
 *
 *  Created on: 2010/12/07
 *      Author: Axiohelix
 */

#ifndef TaxonomyNode_H_
#define TaxonomyNode_H_

#include	<string>
#include	<list>

#include	"Const.h"

/**
 * NCBIのTaxonomyノード情報を格納するオブジェクト.
 */
class TaxonomyNode {
	private:
		/* TaxonomyID */
		int m_id;
		/* Taxonomy name */
		std::string m_name;
		/* 親のTaxonomyID */
		int m_parentId;
		/* ルートのTaxonomyID */
		int m_rootId;
		/* ルートのTaxonomy name */
		std::string m_rootName;
		/* 門のTaxonomy ID */
		int m_phylumId;
		/* 門のTaxonomy name */
		std::string m_phylumName;
		/* 属のTaxonomy ID */
		int m_genusId;
		/* 属のTaxonomy name */
		std::string m_genusName;

		/* rank */
		std::string m_rank;
		/* lineageを保持するlist. */
		/* 中身は逆順に入っている. */
		std::list<std::string> m_lineageList;

	public:
		/* コンストラクタ・デストラクタ */
		TaxonomyNode() {
			m_id = 0;
		}

		/**
		 * コンストラクタ
		 * Taxonomy ID, parentTaxonomyId, rankを与えてノードを生成する.
		 * これは、nodes.dmpから得られる一次情報.
		 */
		TaxonomyNode(const int id, const int parentId, const std::string rank) {
			m_id = id;
			m_parentId = parentId;
			m_rank = rank;

			/* ルート, 門, 属はオブジェクト生成時点では未定 */
			m_rootId = -1;
			m_phylumId = -1;
			m_genusId = -1;
		}

		virtual ~TaxonomyNode() { }

		/* setter / getter */
		inline void SetId(const int id) { m_id = id; }
		inline const int GetId() { return m_id; }

		inline void SetName(const std::string name) { m_name = name; }
		inline const std::string GetName() { return m_name; }

		inline void SetParentId(const int parentId) { m_parentId = parentId; }
		inline const int GetParentId() { return m_parentId; }

		inline void SetRootId(const int rootId) { m_rootId = rootId; }
		inline const int GetRootId() { return m_rootId; }

		inline void SetRootName(const std::string name) { m_rootName = name; }
		inline const std::string GetRootName() { return m_rootName; }

		inline void SetPhylumId(const int phylumId) { m_phylumId = phylumId; }
		inline const int GetPhylumId() { return m_phylumId; }

		inline void SetPhylumName(const std::string name) { m_phylumName = name; }
		inline const std::string GetPhylumName() { return m_phylumName; }

		inline void SetGenusId(const int genusId) { m_genusId = genusId; }
		inline const int GetGenusId() { return m_genusId; }

		inline void SetGenusName(const std::string name) { m_genusName = name; }
		inline const std::string GetGenusName() { return m_genusName; }

		inline void SetRank(const std::string rank) { m_rank = rank; }
		inline const std::string GetRank() { return m_rank; }

		inline void SetLineageList(const std::list<std::string> lineageList) { m_lineageList = lineageList; }
		inline const std::list<std::string> GetLineageList() { return m_lineageList; }

		/* 情報出力用簡易メソッド */
		const std::string GetInformation(bool, bool);
		/* lineage出力用メソッド */
		const std::string getLineage(std::string);
};

#endif /* TaxonomyNode_H_ */
